# -*- coding: UTF-8 -*-

class ConnectorError(BaseException):
    """Exception raised if the informations supplied
    to initiate the connector are wrong.
    """
    pass

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
